==> brainsurf-0.0.3.tar.gz: brainsurf-0.0.3/brainsurf/preprocessing/filtering.py <==
import numpy as np
from scipy.signal import butter, lfilter, iirnotch

def butter_bandpass_filter(data, lowcut, highcut, fs, order):
    """
    Apply a Butterworth band-pass filter to the input data.

    Parameters:
        data (array-like): Input data to be filtered.
        lowcut (float): Lower cut-off frequency of the filter.
        highcut (float): Upper cut-off frequency of the filter.
        fs (float): Sampling frequency of the data.
        order (int): Order of the Butterworth filter.

    Returns:
        array-like: Filtered data.

    """
    nyq = 0.5 * fs
    low = lowcut / nyq
    high = highcut / nyq
    b, a = butter(order, [low, high], btype='band')
    y = lfilter(b, a, data)
    return y

def notch_filter(data, fs, freqs, q):
    """
    Apply an IIR notch filter to remove specific frequencies from the input data.

    Parameters:
        data (array-like): Input data to be filtered.
        fs (float): Sampling frequency of the data.
        freqs (float or array-like): Frequency or frequencies to be removed.
        q (float): Quality factor of the notch filter.

    Returns:
        array-like: Filtered data.

    """
    b, a = iirnotch(freqs, q, fs)
    y = lfilter(b, a, data)
    return y

def lowpass_filter(data, cutoff_freq, fs, order):
    """
    Apply a low-pass filter to the input data.

    Parameters:
        data (array-like): Input data to be filtered.
        cutoff_freq (float): Cut-off frequency of the filter.
        fs (float): Sampling frequency of the data.
        order (int): Order of the Butterworth filter.

    Returns:
        array-like: Filtered data.

    """
    nyq = 0.5 * fs
    cutoff = cutoff_freq / nyq
    b, a = butter(order, cutoff, btype='low')
    y = lfilter(b, a, data)
    return y

def highpass_filter(data, cutoff_freq, fs, order):
    """
    Apply a high-pass filter to the input data.

    Parameters:
        data (array-like): Input data to be filtered.
        cutoff_freq (float): Cut-off frequency of the filter.
        fs (float): Sampling frequency of the data.
        order (int): Order of the Butterworth filter.

    Returns:
        array-like: Filtered data.

    """
    nyq = 0.5 * fs
    cutoff = cutoff_freq / nyq
    b, a = butter(order, cutoff, btype='high')
    y = lfilter(b, a, data)
    return y

def comb_filter(data, delay, gain):
    """
    Apply a comb filter to the input data.

    Parameters:
        data (array-like): Input data to be filtered.
        delay (int): Delay value for the comb filter.
        gain (float): Gain value for the comb filter.

    Returns:
        array-like: Filtered data.

    """
    y = np.zeros_like(data)
    for i in range(delay, len(data)):
        y[i] = data[i] + gain * data[i - delay]
    return y

def adaptive_filter(data, reference, order):
    """
    Apply an adaptive filter to the input data.

    Parameters:
        data (array-like): Input data to be filtered.
        reference (array-like): Reference data for the adaptive filter.
        order (int): Order of the adaptive filter.

    Returns:
        array-like: Filtered data.

    """
    y = np.zeros_like(data)
    w = np.zeros(order)
    for i in range(order, len(data)):
        x = data[i-order:i][::-1]
        y[i] = np.dot(w, x)
        e = reference[i] - y[i]
        w += 0.01 * e * x
    return y

def kalman_filter(data, measurement_noise, process_noise):
    """
    Apply a Kalman filter to the input data.

    Parameters:
        data (array-like): Input data to be filtered.
        measurement_noise (float): Measurement noise covariance.
        process_noise (float): Process noise covariance.

    Returns:
        array-like: Filtered data.

    """
    n = len(data)
    x = np.zeros(n)
    P = np.zeros(n)
    x[0] = data[0]
    P[0] = measurement_noise
    for i in range(1, n):
        x_priori = x[i-1]
        P_priori = P[i-1] + process_noise
        K = P_priori / (P_priori + measurement_noise)
        x[i] = x_priori + K * (data[i] - x_priori)
        P[i] = (1 - K) * P_priori
    return x

==> brainsurf-7.0.0.tar.gz: brainsurf-7.0.0/brainsurf/preprocessing/filtering.py <==
import numpy as np
from scipy.signal import butter, lfilter, iirnotch

def butter_bandpass_filter(data, lowcut, highcut, fs, order):
    """
    Apply a Butterworth band-pass filter to the input data.

    Parameters:
        data (array-like): Input data to be filtered.
        lowcut (float): Lower cut-off frequency of the filter.
        highcut (float): Upper cut-off frequency of the filter.
        fs (float): Sampling frequency of the data.
        order (int): Order of the Butterworth filter.

    Returns:
        array-like: Filtered data.

    """
    nyq = 0.5 * fs
    low = lowcut / nyq
    high = highcut / nyq
    b, a = butter(order, [low, high], btype='band')
    y = lfilter(b, a, data)
    return y

def notch_filter(data, fs, freqs, q):
    """
    Apply an IIR notch filter to remove specific frequencies from the input data.

    Parameters:
        data (array-like): Input data to be filtered.
        fs (float): Sampling frequency of the data.
        freqs (float or array-like): Frequency or frequencies to be removed.
        q (float): Quality factor of the notch filter.

    Returns:
        array-like: Filtered data.

    """
    b, a = iirnotch(freqs, q, fs)
    y = lfilter(b, a, data)
    return y

def lowpass_filter(data, cutoff_freq, fs, order):
    """
    Apply a low-pass filter to the input data.

    Parameters:
        data (array-like): Input data to be filtered.
        cutoff_freq (float): Cut-off frequency of the filter.
        fs (float): Sampling frequency of the data.
        order (int): Order of the Butterworth filter.

    Returns:
        array-like: Filtered data.

    """
    nyq = 0.5 * fs
    cutoff = cutoff_freq / nyq
    b, a = butter(order, cutoff, btype='low')
    y = lfilter(b, a, data)
    return y

def apply_highpass_filter(raw, highpass_freq):
    """
    Apply a high-pass filter to the raw data.

    Parameters:
        raw (Raw object): The raw data to be filtered.
        highpass_freq (float): The cut-off frequency for the high-pass filter.

    Returns:
        Raw object: The filtered raw data.

    """
    raw.filter(l_freq=highpass_freq, h_freq=None)
    return raw

def apply_lowpass_filter(raw, lowpass_freq):
    """
    Apply a low-pass filter to the raw data.

    Parameters:
        raw (Raw object): The raw data to be filtered.
        lowpass_freq (float): The cut-off frequency for the low-pass filter.

    Returns:
        Raw object: The filtered raw data.

    """
    raw.filter(l_freq=None, h_freq=lowpass_freq)
    return raw


def highpass_filter(data, cutoff_freq, fs, order):
    """
    Apply a high-pass filter to the input data.

    Parameters:
        data (array-like): The input data to be filtered.
        cutoff_freq (float): The cut-off frequency of the filter.
        fs (float): The sampling frequency of the data.
        order (int): The order of the Butterworth filter.

    Returns:
        array-like: The filtered data.

    """
    nyq = 0.5 * fs
    cutoff = cutoff_freq / nyq
    b, a = butter(order, cutoff, btype='high')
    y = lfilter(b, a, data)
    return y

def comb_filter(data, delay, gain):
    """
    Apply a comb filter to the input data.

    Parameters:
        data (array-like): The input data to be filtered.
        delay (int): The delay parameter of the comb filter.
        gain (float): The gain parameter of the comb filter.

    Returns:
        array-like: The filtered data.

    """
    y = np.zeros_like(data)
    for i in range(delay, len(data)):
        y[i] = data[i] + gain * data[i - delay]
    return y

def adaptive_filter(data, reference, order):
    """
    Apply an adaptive filter to the input data.

    Parameters:
        data (array-like): The input data to be filtered.
        reference (array-like): The reference signal for the adaptive filter.
        order (int): The order of the adaptive filter.

    Returns:
        array-like: The filtered data.

    """
    y = np.zeros_like(data)
    w = np.zeros(order)
    for i in range(order, len(data)):
        x = data[i-order:i][::-1]
        y[i] = np.dot(w, x)
        e = reference[i] - y[i]
        w += 0.01 * e * x
    return y

def kalman_filter(data, measurement_noise, process_noise):
    """
    Apply a Kalman filter to the input data.

    Parameters:
        data (array-like): The input data to be filtered.
        measurement_noise (float): The measurement noise of the Kalman filter.
        process_noise (float): The process noise of the Kalman filter.

    Returns:
        array-like: The filtered data.

    """
    n = len(data)
    x = np.zeros(n)
    P = np.zeros(n)
    x[0] = data[0]
    P[0] = measurement_noise
    for i in range(1, n):
        x_priori = x[i-1]
        P_priori = P[i-1] + process_noise
        K = P_priori / (P_priori + measurement_noise)
        x[i] = x_priori + K * (data[i] - x_priori)
        P[i] = (1 - K) * P_priori
    return x


def apply_spatial_filter(epochs):
    """
    Apply a spatial filter to the input epochs.

    Parameters:
        epochs (Epochs object): The epochs data to be filtered.

    Returns:
        Epochs object: The spatially filtered epochs.

    """
    spatial_filtered_epochs = epochs  # Placeholder, implement as required
    return spatial_filtered_epochs
